@@ -42,6 +42,7 @@
 #include <fstream>
 #include <iomanip>
 #include <memory>
+#include <cmath>
 
 using namespace std;
 
@@ -56,8 +57,20 @@ using namespace std;
 #include <Bpp/Seq/SiteTools.h>
 #include <Bpp/Seq/CodonSiteTools.h>
 #include <Bpp/Seq/Alphabet/Alphabet.h>
+#include <Bpp/Seq/Alphabet/AlphabetTools.h>
 #include <Bpp/Seq/App/SequenceApplicationTools.h>
 
+// From bpp-phyl:
+#include <Bpp/Phyl/Tree.h>
+#include <Bpp/Phyl/Model/Nucleotide/K80.h>
+#include <Bpp/Phyl/Model/Codon/YN98.h>
+#include <Bpp/Phyl/Model/RateDistribution/ConstantRateDistribution.h>
+#include <Bpp/Phyl/Distance/DistanceEstimation.h>
+#include <Bpp/Phyl/Distance/BioNJ.h>
+#include <Bpp/Phyl/Likelihood/DRHomogeneousTreeLikelihood.h>
+#include <Bpp/Phyl/Likelihood/MarginalAncestralStateReconstruction.h>
+#include <Bpp/Phyl/App/PhylogeneticsApplicationTools.h>
+
 // From bpp-popgen
 #include <Bpp/PopGen/PolymorphismSequenceContainer.h>
 #include <Bpp/PopGen/PolymorphismSequenceContainerTools.h>
@@ -140,6 +153,7 @@ int main(int args, char** argv)
       if (ApplicationTools::parameterExists("input.sequence.outgroup.name", bpppopstats.getParams())) {
         vector<string> outgroups = ApplicationTools::getVectorParameter<string>("input.sequence.outgroup.name", bpppopstats.getParams(), ',', "");
         for (auto g : outgroups) {
+          ApplicationTools::displayResult("Sequence from outgroup", g);
           psc->setAsOutgroupMember(g);
         }
       }
@@ -182,7 +196,104 @@ int main(int args, char** argv)
     }
     ApplicationTools::displayResult("Number of sequences in ingroup", pscIn->getNumberOfSequences());
     ApplicationTools::displayResult("Number of sequences in outgroup", pscOut.get() ? pscOut->getNumberOfSequences() : 0);
+   
+    // Shall we estimate some parameters first?
     
+    bool estimateTsTv = ApplicationTools::getBooleanParameter("estimate.kappa", bpppopstats.getParams(), false, "", false, 1);
+    double kappa = 1;
+    double omega = -1;
+    
+    bool estimateAncestor = ApplicationTools::getBooleanParameter("estimate.ancestor", bpppopstats.getParams(), false, "", false, 1);
+    if (estimateAncestor & ! pscOut)
+      throw Exception("Error: an outgroup sequence is needed for estimating ancestral states.");
+
+    bool fitModel = estimateTsTv || estimateAncestor;
+
+    // Fit a model for later use:
+    unique_ptr<Tree> tree;
+    unique_ptr<SubstitutionModel> model;
+    unique_ptr<DiscreteDistribution> rDist;
+    DRTreeLikelihood* treeLik = nullptr;
+    unique_ptr<Sequence> ancestralSequence;
+    if (fitModel) {
+      // Get the alignment:
+      bool sampleIngroup = ApplicationTools::getBooleanParameter("estimate.sample_ingroup", bpppopstats.getParams(), true);
+      size_t sampleIngroupSize = 0;
+      if (sampleIngroup) {
+        sampleIngroupSize = ApplicationTools::getParameter<size_t>("estimate.sample_ingroup.size", bpppopstats.getParams(), 10);
+        if (sampleIngroupSize > pscIn->getNumberOfSequences()) {
+          ApplicationTools::displayWarning("Sample size higher than number of sequence. No sampling performed.");
+          sampleIngroup = false;
+        }
+      }
+      unique_ptr<AlignedSequenceContainer> aln;
+      if (sampleIngroup) {
+        ApplicationTools::displayResult("Nb of ingroup sequences for model fitting", sampleIngroupSize);
+        aln.reset(new AlignedSequenceContainer(pscIn->getAlphabet()));
+        vector<string> selection(sampleIngroupSize);
+        RandomTools::getSample(pscIn->getSequencesNames(), selection, false); 
+        SequenceContainerTools::getSelectedSequences(*pscIn, selection ,*aln);
+      } else {
+        aln.reset(new AlignedSequenceContainer(*pscIn));
+      }
+      if (pscOut) {
+        aln->addSequence(pscOut->getSequence(0)); //As for now, we only consider one sequence as outgroup, the first one.
+      }
+
+      // Get a tree:
+      string treeOpt = ApplicationTools::getStringParameter("input.tree.method", bpppopstats.getParams(), "bionj", "");
+      if (codonAlphabet) {
+        unique_ptr<FrequenciesSet> freqSet(new FixedCodonFrequenciesSet(gCode.get()));
+        model.reset(new YN98(gCode.get(), freqSet.release()));
+      } else {
+        model.reset(new K80(&AlphabetTools::DNA_ALPHABET));
+      } //Note: proteins not supported!
+      rDist.reset(new ConstantRateDistribution()); 
+      if (treeOpt == "user") {
+        tree.reset(PhylogeneticsApplicationTools::getTree(bpppopstats.getParams()));
+      } else if (treeOpt == "bionj") {
+        ApplicationTools::displayTask("Estimating distance matrix", true);
+        //DistanceEstimation distEstimation(model->clone(), rDist->clone(), aln.get(), 1, false);
+        //distEstimation.computeMatrix();
+        //unique_ptr<DistanceMatrix> matrix(distEstimation.getMatrix());
+        unique_ptr<DistanceMatrix> matrix(SiteContainerTools::computeSimilarityMatrix(*aln, true, SiteContainerTools::SIMILARITY_NOGAP, true));
+        
+        ApplicationTools::displayTaskDone();
+        ApplicationTools::displayTask("Computing BioNJ tree", true);
+        BioNJ bionj(false, true);
+        bionj.setDistanceMatrix(*matrix);
+        bionj.computeTree();
+        ApplicationTools::displayTaskDone();
+        tree.reset(bionj.getTree());
+      } else {
+        throw Exception("Invalid input.tree.method. Should be either 'user' or 'bionj'.");
+      }
+
+      // Create a likelihood object:
+      treeLik = new DRHomogeneousTreeLikelihood(*tree, *aln, model.get(), rDist.get());
+      treeLik->initialize();
+      if (std::isinf(treeLik->getValue()))
+        throw Exception("Error: null likelihood. Possible cause: stop codon or numerical underflow (too many sequences).");
+      // Optimize parameters:
+      treeLik = dynamic_cast<DRTreeLikelihood*>(PhylogeneticsApplicationTools::optimizeParameters(treeLik, treeLik->getParameters(), bpppopstats.getParams(), "", true, true, 2));
+
+      // Get kappa:              
+      if (estimateTsTv) {
+        kappa = model->getParameter("kappa").getValue();
+        ApplicationTools::displayResult("Transition / transversions ratio", kappa);
+      }
+      if (estimateAncestor) {
+        MarginalAncestralStateReconstruction asr(treeLik);
+        int outgroupId = tree->getLeafId(pscOut->getSequence(0).getName());
+        ancestralSequence.reset(asr.getAncestralSequenceForNode(tree->getFatherId(outgroupId)));
+      }
+      if (codonAlphabet) {
+        omega = model->getParameter("omega").getValue();
+      }
+    }
+    if (treeLik)
+      delete treeLik; //Not needed anymore.
+
     // +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
     
     // Compute statistics
@@ -312,7 +423,6 @@ int main(int args, char** argv)
             *cLog << "fuLiFstarSegSit" << (toolCounter[cmdName] > 1 ? TextTools::toString(toolCounter[cmdName]) : "") << " = " << flFstar << endl;
         }
       }
-
       // +-----------+
       // | PiN / PiS |
       // +-----------+
@@ -323,20 +433,61 @@ int main(int args, char** argv)
         }
         double piS = SequenceStatistics::piSynonymous(*pscIn, *gCode);
         double piN = SequenceStatistics::piNonSynonymous(*pscIn, *gCode);
-        double nbS = SequenceStatistics::meanNumberOfSynonymousSites(*pscIn, *gCode);
-        double nbN = SequenceStatistics::meanNumberOfNonSynonymousSites(*pscIn, *gCode);
+        double nbS = SequenceStatistics::meanNumberOfSynonymousSites(*pscIn, *gCode, kappa);
+        double nbN = SequenceStatistics::meanNumberOfNonSynonymousSites(*pscIn, *gCode, kappa);
         double r = (piN / nbN) / (piS / nbS);
         ApplicationTools::displayResult("PiN:", piN);
         ApplicationTools::displayResult("PiS:", piS);
         ApplicationTools::displayResult("#N:", nbN);
         ApplicationTools::displayResult("#S:", nbS);
         ApplicationTools::displayResult("PiN / PiS (corrected for #N and #S):", r);
+        if (fitModel) {
+          ApplicationTools::displayResult("Omega (YN98 model):", omega);
+        }
+
         if (logFile != "none") {
           *cLog << "# PiN and PiS" << endl;
           *cLog << "PiN" << (toolCounter[cmdName] > 1 ? TextTools::toString(toolCounter[cmdName]) : "") << " = " << piN << endl;
           *cLog << "PiS" << (toolCounter[cmdName] > 1 ? TextTools::toString(toolCounter[cmdName]) : "") << " = " << piS << endl;
           *cLog << "NbN" << (toolCounter[cmdName] > 1 ? TextTools::toString(toolCounter[cmdName]) : "") << " = " << nbN << endl;
           *cLog << "NbS" << (toolCounter[cmdName] > 1 ? TextTools::toString(toolCounter[cmdName]) : "") << " = " << nbS << endl;
+          if (fitModel) {
+            *cLog << "Omega" << (toolCounter[cmdName] > 1 ? TextTools::toString(toolCounter[cmdName]) : "") << " = " << omega << endl;
+          }
+        }
+      }
+
+      // +---------+
+      // | dN / dS |
+      // +---------+
+      else if (cmdName == "dN_dS")
+      {
+        if (!codonAlphabet) {
+          throw Exception("dN_dS can only be used with a codon alignment. Check the input alphabet!");
+        }
+        //Get consensus sequences:
+        unique_ptr<SiteContainer> alnIn(pscIn->toSiteContainer());
+        unique_ptr<SiteContainer> alnOut(pscOut->toSiteContainer());
+        unique_ptr<Sequence> consensusIn(SiteContainerTools::getConsensus(*alnIn, "consIn", true, false));
+        unique_ptr<Sequence> consensusOut(SiteContainerTools::getConsensus(*alnOut, "consOut", true, false));
+        unique_ptr<AlignedSequenceContainer> alnCons(new AlignedSequenceContainer(codonAlphabet));
+        alnCons->addSequence(*consensusIn);
+        alnCons->addSequence(*consensusOut);
+        unique_ptr<FrequenciesSet> freqSetDiv(new FixedCodonFrequenciesSet(gCode.get()));
+        YN98* modelDiv = new YN98(gCode.get(), freqSetDiv.release());
+        DiscreteDistribution* rDistDiv = new ConstantRateDistribution(); 
+        DistanceEstimation distEstimation(modelDiv, rDistDiv, alnCons.get(), 0, false);
+        distEstimation.setAdditionalParameters(modelDiv->getIndependentParameters());
+        distEstimation.computeMatrix();
+        unique_ptr<DistanceMatrix> matrix(distEstimation.getMatrix());
+        ApplicationTools::displayResult("Yang and Nielsen's Omega (dN/dS):", modelDiv->getParameter("omega").getValue());
+        ApplicationTools::displayResult("Yang and Nielsen's Kappa:", modelDiv->getParameter("kappa").getValue());
+        ApplicationTools::displayResult("Yang and Nielsen's Distance:", (*matrix)(1,0));
+        if (logFile != "none") {
+          *cLog << "# dN and dS (Yang and Nielsen's 1998 substitution model)" << endl;
+          *cLog << "OmegaDiv" << (toolCounter[cmdName] > 1 ? TextTools::toString(toolCounter[cmdName]) : "") << " = " << modelDiv->getParameter("omega").getValue() << endl;
+          *cLog << "KappaDiv" << (toolCounter[cmdName] > 1 ? TextTools::toString(toolCounter[cmdName]) : "") << " = " << modelDiv->getParameter("kappa").getValue() << endl;
+          *cLog << "DistanceDiv" << (toolCounter[cmdName] > 1 ? TextTools::toString(toolCounter[cmdName]) : "") << " = " << (*matrix)(1,0) << endl;
         }
       }
 
@@ -348,7 +499,7 @@ int main(int args, char** argv)
         if (!codonAlphabet) {
           throw Exception("MacDonald-Kreitman test can only be performed on a codon alignment. Check the input alphabet!");
         }
-        if (!pscOut.get()) {
+        if (!pscOut) {
           throw Exception("MacDonald-Kreitman test requires at least one outgroup sequence.");
         }
         vector<unsigned int> mktable = SequenceStatistics::mkTable(*pscIn, *pscOut, *gCode);
@@ -375,30 +526,99 @@ int main(int args, char** argv)
         if (path == "none") throw Exception("You must specify an ouptut file for CodonSiteStatistics"); 
         ApplicationTools::displayResult("Site statistics output to:", path);
         ofstream out(path.c_str(), ios::out);
-        out << "Site\tIsComplete\tNbAlleles\tMinorAlleleFrequency\tMajorAlleleFrequency\tMinorAllele\tMajorAllele";
-        bool outgroup = (psc->hasOutgroup() && pscOut->getNumberOfSequences() == 1);
+        out << "Site\tMissingDataFrequency\tNbAlleles\tMinorAlleleFrequency\tMajorAlleleFrequency\tMinorAllele\tMajorAllele";
+        out << "\tMeanNumberSynPos\tIsSynPoly\tIs4Degenerated\tPiN\tPiS";
+        bool outgroup = (pscOut && pscOut->getNumberOfSequences() == 1);
         if (outgroup) {
           out << "\tOutgroupAllele";
         }
-        out << "\tMeanNumberSynPos\tIsSynPoly\tIs4Degenerated\tPiN\tPiS" << endl;
+        if (estimateAncestor) {
+          out << "\tAncestralAllele";
+        }
+        if (outgroup) {
+          out << "\tMeanNumberSynPosDiv\tdN\tdS";
+        }
+        out << endl;
+
         unique_ptr<SiteContainer> sites(pscIn->toSiteContainer());
         for (size_t i = 0; i < sites->getNumberOfSites(); ++i) {
           const Site& site = sites->getSite(i);
+          map<int, size_t> counts;
+          SymbolListTools::getCounts(site, counts);
+          size_t minFreq = site.size() + 1;
+          size_t maxFreq = 0;
+          int minState = -1;
+          int maxState = -1;
+          size_t nbAlleles = 0;
+          size_t nbMissing = 0;
+          for (map<int, size_t>::iterator it = counts.begin(); it != counts.end(); it++)
+          {
+            if (!alphabet->isUnresolved(it->first)
+              && !alphabet->isGap(it->first)) {
+              nbAlleles++; 
+              if (it->second != 0) {
+                if (it->second < minFreq) {
+                  minFreq = it->second;
+                  minState = it->first;        
+                }
+                if (it->second > maxFreq) {
+                  maxFreq = it->second;
+                  maxState = it->first;
+                }
+              }
+            } else {
+              nbMissing += it->second;            
+            }
+          }
+          
           out << site.getPosition() << "\t";
-          out << SiteTools::isComplete(site) << "\t";
-          out << SiteTools::getNumberOfDistinctCharacters(site) << "\t";
-          out << SiteTools::getMinorAlleleFrequency(site) << "\t";
-          out << SiteTools::getMajorAlleleFrequency(site) << "\t";
-          out << alphabet->intToChar(SiteTools::getMinorAllele(site)) << "\t";
-          out << alphabet->intToChar(SiteTools::getMajorAllele(site)) << "\t";
-          if (outgroup) {
-           out << pscOut->getSequence(0).getChar(i) << "\t"; 
+          out << nbMissing << "\t";
+          out << nbAlleles << "\t";
+          out << minFreq << "\t";
+          out << maxFreq << "\t";
+          out << alphabet->intToChar(minState) << "\t";
+          out << alphabet->intToChar(maxState) << "\t";
+          if (estimateAncestor) {
+            out << CodonSiteTools::numberOfSynonymousPositions(ancestralSequence->getValue(i), *gCode, kappa) << "\t";
+          } else {
+            out << CodonSiteTools::meanNumberOfSynonymousPositions(site, *gCode, kappa) << "\t";
           }
-          out << CodonSiteTools::meanNumberOfSynonymousPositions(site, *gCode) << "\t";
           out << CodonSiteTools::isSynonymousPolymorphic(site, *gCode) << "\t";
           out << CodonSiteTools::isFourFoldDegenerated(site, *gCode) << "\t";
           out << CodonSiteTools::piNonSynonymous(site, *gCode) << "\t";
-          out << CodonSiteTools::piSynonymous(site, *gCode) << endl;
+          out << CodonSiteTools::piSynonymous(site, *gCode);
+          if (outgroup) {
+           out << "\t" << pscOut->getSequence(0).getChar(i); 
+          }
+          if (estimateAncestor) {
+           out << "\t" << ancestralSequence->getChar(i); 
+          }
+          if (outgroup) {
+            //Add divergence
+            int outgroupState = pscOut->getSequence(0)[i];
+            if (codonAlphabet->isUnresolved(outgroupState) || codonAlphabet->isGap(outgroupState)) {
+              out << "\tNA\tNA\tNA";
+            } else {
+              //Average over outgroup (Note: minState and maxState are identical in this case)
+              out << "\t" << (CodonSiteTools::numberOfSynonymousPositions(outgroupState, *gCode, kappa) +
+                                     CodonSiteTools::numberOfSynonymousPositions(minState, *gCode, kappa)) / 2.;
+              if (nbAlleles == 1) {
+                //Compare with outgroup:
+                if (site[0] == outgroupState) {
+                  out << "\t0\t0";
+                } else {
+                  //This is a real substitution:
+                  double nt = static_cast<double>(CodonSiteTools::numberOfDifferences(outgroupState, minState, *codonAlphabet));
+                  double ns = CodonSiteTools::numberOfSynonymousDifferences(outgroupState, minState, *gCode); 
+                  out << "\t" << (nt - ns) << "\t" << ns;
+                }
+              } else {
+                //Site is polymorphic, this is not a substitution    
+                out << "\t0\t0";
+              }
+            }
+          }
+          out << endl;
         }
       }
       
